/*
 * Copyright 2008-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wamblee.glassfish.statistics;

import org.wamblee.glassfish.auth.cache.impl.SimpleExpiryCache.Clock;

/**
 * Represents a stastic with success and fail count and total duration.
 * 
 * @author Erik Brakkee
 * 
 */
public class Statistic {

    public static interface Clock {
        long currentTimeMillis();
    }

    public static class SystemClock implements Clock {
        @Override
        public long currentTimeMillis() {
            return System.currentTimeMillis();
        }
    }

    public class Measurement {
        long tstart;

        public Measurement() {
            tstart = clock.currentTimeMillis();
        }

        public void stop(boolean aSuccess) {
            if (aSuccess) {
                Statistic.this.succeed(this);
            } else {
                Statistic.this.fail(this);
            }
        }

        public long getDuration() {
            return clock.currentTimeMillis() - tstart;
        }
    }

    private Clock clock;

    private long duration;
    private long succeedCount;
    private long failCount;

    /**
     * Constructs a statistic.
     * 
     * @param aClock
     *            Clock.
     */
    public Statistic(Clock aClock) {
        clock = aClock;
    }

    public Measurement start() {
        return new Measurement();
    }

    public synchronized void clear() {
        duration = 0;
        succeedCount = 0;
        failCount = 0;
    }

    private synchronized void succeed(Measurement aMeasurement) {
        duration += aMeasurement.getDuration();
        succeedCount++;
    }

    private synchronized void fail(Measurement aMeasurement) {
        duration += aMeasurement.getDuration();
        failCount++;
    }

    public long getDurationMillis() {
        return duration;
    }

    public long getSucceedCount() {
        return succeedCount;
    }

    public long getFailCount() {
        return failCount;
    }
}
